#!/usr/bin/env python3
#-*- coding: utf8 -*-

import datetime
import decimal

import uno

from com.sun.star.beans import PropertyValue
from com.sun.star.connection import NoConnectException
from dateutil.relativedelta import relativedelta
from unohelper import systemPathToFileUrl

CONNECT_MSG = """
Need to start Libreffice! Use a command like:
    libreoffice -accept="socket,host=localhost,port=2002;urp;"
"""


filename = "/tmp/Distrib.ods"
local = uno.getComponentContext()
resolver = local.ServiceManager.createInstanceWithContext(
        'com.sun.star.bridge.UnoUrlResolver', local)

try:
    ctx = resolver.resolve('uno:socket,host=localhost,port=2002;'
                           'urp;StarOffice.ComponentContext')
except NoConnectException:
    print(CONNECT_MSG)
    exit()

smgr = ctx.ServiceManager
desktop = smgr.createInstanceWithContext('com.sun.star.frame.Desktop',
                                         ctx)
in_props = (PropertyValue("Hidden", 0, True, 0),)
document = desktop.loadComponentFromURL(systemPathToFileUrl(filename),
                                        '_blank', 0, in_props)


# prepare month grouping
today = datetime.date.today()
def _gen_gr(quant):
    """Generate the groups."""
    gr = set()
    for m in range(1, quant + 1):
        d = today - relativedelta(months=m)
        gr.add((d.year, d.month))
    return gr

GR_1M = _gen_gr(1)
GR_3M = _gen_gr(3)
GR_1Y = _gen_gr(12)
MONTH_MARKS = [
    "mes", "enero", "febrero", "marzo", "abril", "mayo", "junio", "julio",
    "agosto", "septiembre", "octubre", "noviembre", "diciembre", "bono",
    "bonus", "aguinaldo", "especial",
]

def _get_row(sheet, colbase, row):
    """Return rows of three columns starting at some point."""
    while True:
        c0 = sheet.getCellByPosition(colbase, row).getString()
        c1 = sheet.getCellByPosition(colbase + 1, row).getString()
        c2 = sheet.getCellByPosition(colbase + 2, row).getString()
        if not c0:
            yield (None, None, None)
            row += 1
            continue

        try:
            # date
            d, m, y = map(int, c0.split("/"))
            date = datetime.date(2000 + y, m, d)

            # explanation
            expl = c1

            # amount
            amount = decimal.Decimal(c2.replace(",", "."))
        except:
            print("ERROR: bad data in row", row)
            raise

        yield (date, expl, amount)
        row += 1


def get_columns(sheet):
    """Get all the columns for each sheet."""
    col = 0
    while True:
        data = []

        # get name
        cell = sheet.getCellByPosition(col + 1, 0)
        colname = cell.getFormula()
        if colname == "":
            # no more columns!
            return

        # get total and skip empty lines
        total = sheet.getCellByPosition(col + 2, 1).getString()
        assert total != ""
        total = decimal.Decimal(total.replace(",", "."))
        assert sheet.getCellByPosition(col, 1).getFormula() == ""
        assert sheet.getCellByPosition(col, 2).getFormula() == ""

        # grab main chunk
        rowgen = _get_row(sheet, col, 3)
        for date, item, val in rowgen:
            if not date:
                break
            data.append((date, item, val))
        rown = len(data) + 3

        # find second chunk, if any (explore some rows)
        for _ in range(500):
            cell = sheet.getCellByPosition(col, rown).getString()
            if cell:
                # found!
                break
            rown += 1
        else:
            # nothing found
            yield (colname, total, data)
            col += 4
            continue

        # grab legacy chunk
        lastyear = datetime.date.today() - datetime.timedelta(days=365)
        while True:
            cell = sheet.getCellByPosition(col, rown).getString()
            if cell:
                # date found, check if it's a year ago
                d, m, y = map(int, cell.split("/"))
                date = datetime.date(2000 + y, m, d)
                if date < lastyear:
                    rown += 50
                    continue

            # undo last jump, and start from here
            rown -= 50
            break

        rowgen = _get_row(sheet, col, rown)
        legacy = []
        for date, item, val in rowgen:
            if not date:
                break
            legacy.append((date, item, val))

        yield (colname, total, legacy + data)
        col += 4


def proc_col(data, total):
    """Process the column."""
    gr_1m = 0
    gr_3m = 0
    gr_1y = 0
    for date, item, amount in data:
        item = item.lower()
        if amount > 0:
            if any(mark in item for mark in MONTH_MARKS):
                continue
        key = (date.year, date.month)
        if key in GR_1M:
            gr_1m += amount
        if key in GR_3M:
            gr_3m += amount
        if key in GR_1Y:
            gr_1y += amount
            if date.day == 1 and amount > 0:
                print("WARNING: Month?", repr(item))
    return gr_1m, gr_3m, gr_1y


sheets = u"Personas Hogar Servicios Auto".split()
for sheet_name in sheets:
    sheet = document.Sheets.getByName(sheet_name)
    print(" " * 22 + u"     Mes     3m    Año")
    for name, total, rawdata in get_columns(sheet):
        try:
            data = proc_col(rawdata, total)
        except Exception as err:
            print("%s parsing col %r: %s" % (err.__class__, name, err))
            exit()
        gr_1m, gr_3m, gr_1y = data
        print("%-22s:  %5d  %5d  %5d" % (name, -gr_1m, -gr_3m / 3, -gr_1y / 12))
    print()
